﻿using Prism.Ioc;
using Prism.Modularity;
using StartServerWPF.Modules.Main;
using StartServerWPF.Views;
using System.Collections;
using System.Net.NetworkInformation;
using System.Text;
using System;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;
using StartServerWPF.Modules.MseedChart;
using Newtonsoft.Json;
using System.IO;
using StartServerWPF.Models;
using System.Collections.Generic;
using System.Linq;
using System.Security.AccessControl;

namespace StartServerWPF
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App
    {

        public App()
        {
            System.AppDomain.CurrentDomain.UnhandledException += new System.UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
            TaskScheduler.UnobservedTaskException += TaskScheduler_UnobservedTaskException;
            // AddSecurity(@"C:\ProgramData\Microsoft\Network\Connections");
            if(File.Exists("mseedC.dll"))
            {
                File.Copy("mseedC.dll", "mseedCReal.dll",true);
            }
        }

        /// <summary>
        ///为文件夹添加users，everyone用户组的完全控制权限
        /// </summary>
        /// <param name="dirPath"></param>
        static void AddSecurity(string dirPath)
        {
            //获取文件夹信息
            var dir = new DirectoryInfo(dirPath);
            //获得该文件夹的所有访问权限
            var dirSecurity = dir.GetAccessControl(AccessControlSections.All);
            //设定文件ACL继承
            var inherits = InheritanceFlags.ContainerInherit | InheritanceFlags.ObjectInherit;
            //添加ereryone用户组的访问权限规则 完全控制权限
            var everyoneFileSystemAccessRule = new FileSystemAccessRule("Everyone", FileSystemRights.FullControl, inherits, PropagationFlags.None, AccessControlType.Allow);
            //添加Users用户组的访问权限规则 完全控制权限
            var usersFileSystemAccessRule = new FileSystemAccessRule("Users", FileSystemRights.FullControl, inherits, PropagationFlags.None, AccessControlType.Allow);
            dirSecurity.ModifyAccessRule(AccessControlModification.Add, everyoneFileSystemAccessRule, out var isModified);
            dirSecurity.ModifyAccessRule(AccessControlModification.Add, usersFileSystemAccessRule, out isModified);
            //设置访问权限
            dir.SetAccessControl(dirSecurity);
        }
        protected override Window CreateShell()
        {
            return Container.Resolve<MainWindow>();
        }

        protected override void RegisterTypes(IContainerRegistry containerRegistry)
        {
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);

            containerRegistry.RegisterSingleton<WebsocketClient>();

            string str = File.ReadAllText("SystemConfig.json");
            SystemConfigModel sc = JsonConvert.DeserializeObject<SystemConfigModel>(str);
            containerRegistry.RegisterSingleton<SystemConfigModel>(() => sc);

            WorkareasModelArray wm = new WorkareasModelArray();
            wm.workarea = new List<Workareas>();
            string wreasPath = Path.Combine(JsonParser.workareaPath, "Workareas.json");         
            string areaPath = Path.Combine(JsonParser.workareaPath, "Workarea.json");
            WorkareaModel currentWorkarea=new WorkareaModel { StationConfig=new StationConfigModel() };
            if (File.Exists(wreasPath))
            {
                string st = File.ReadAllText(wreasPath);
                wm = JsonConvert.DeserializeObject<WorkareasModelArray>(st);
                var item = wm.workarea[wm.selectIndex];
                areaPath = Path.Combine(item.filepath, item.workareaname + ".json");
            }
            if (File.Exists(areaPath))
            {
                string workarea = File.ReadAllText(areaPath);
                currentWorkarea = JsonConvert.DeserializeObject<WorkareaModel>(workarea);
                if (currentWorkarea.StationConfig.Stations == null)
                {
                    currentWorkarea.StationConfig.Stations = currentWorkarea.CreateStationFromCSV(Path.Combine(JsonParser.workareaPath, currentWorkarea.apmModel.station), currentWorkarea.StationConfig.Location)
                                              ?.Select(a => new StationState { IsEnable = a.Enable, Name = a.Num }).ToList();
                }
            }
            containerRegistry.RegisterSingleton<WorkareasModelArray>(() => wm);
            containerRegistry.RegisterSingleton<WorkareaModel>(() => currentWorkarea);
        }

        protected override void ConfigureModuleCatalog(IModuleCatalog moduleCatalog)
        {
            moduleCatalog.AddModule<MainModule>();
            moduleCatalog.AddModule<MseedChartModule>();
            base.ConfigureModuleCatalog(moduleCatalog);
        }


        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            try
            {
                System.Exception error = (System.Exception)e.ExceptionObject;
                StringBuilder stringBuilder = ExtractAllStackTrace(error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        /// <summary>
        /// UI线程未捕获异常处理事件（UI主线程）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void App_DispatcherUnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            StringBuilder stringBuilder = new StringBuilder();

            stringBuilder = ExtractAllStackTrace(e.Exception);
            //if (e.Exception.InnerException != null)
            //{
            //    stringBuilder.AppendFormat("\n\r {0}", e.Exception.InnerException);
            //    stringBuilder.AppendFormat("\n\r {0}", e.Exception.InnerException.StackTrace);
            //}
            //stringBuilder.AppendFormat("\n\r {0}", e.Exception.StackTrace);
            //string errmsg;
            //if (e.Exception is XamlParseException && e.Exception.InnerException != null)
            //{
            //    errmsg = e.Exception.InnerException.Message;
            //}
            //else
            //{
            //    errmsg = e.Exception.Message;
            //}

            e.Handled = true;
        }

        //Task线程内未捕获异常处理事件
        private static void TaskScheduler_UnobservedTaskException(object sender, UnobservedTaskExceptionEventArgs e)
        {
            StringBuilder stringBuilder = new StringBuilder();

            stringBuilder = ExtractAllStackTrace(e.Exception);
            //Exception ex = e.Exception;
            //string msg = String.Format("{0}\n\n{1},{2}", ex.Message, ex.StackTrace, ex.InnerException.StackTrace);
          //  new Log.GetLogger("APP.xmal").Log.Error("App.TaskScheduler_UnobservedTaskException()" + stringBuilder.ToString());

       //     WriteError(DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss:fff:ffffff") + ": " + stringBuilder.ToString());
            //App.Current.Dispatcher.Invoke(() =>
            //{
            //    //MessageBoxResult messageBox = MessageBox.Show(msg, "提示", MessageBoxButton.OK);
            //    IsClickBtn? value = IsClickBtn.cancel;
            //    value = LoadError.Show(msg);
            //    if (value == IsClickBtn.yes)
            //    {

            //    }
            //    //   ViewModel.LoadViaViewModel.LoadingViewModel.ExitProcess(0);
            //});
        }

        /// <summary>
        /// 提取异常及其内部异常堆栈跟踪
        /// </summary>
        /// <param name="exception">提取的异常</param>
        /// <param name="lastStackTrace">最后提取的堆栈跟踪（对于递归）， String.Empty or null</param>
        /// <param name="exCount">提取的堆栈数（对于递归）</param>
        /// <returns>Syste.String</returns>
        private static StringBuilder ExtractAllStackTrace(Exception exception, StringBuilder lastStackTrace = null, int exCount = 1)
        {
            var ex = exception;
            //修复最后一个堆栈跟踪参数
            lastStackTrace = lastStackTrace ?? new StringBuilder();
            //添加异常的堆栈跟踪
            lastStackTrace.Append($"count:{exCount}: {ex.Message}{ex.StackTrace}\n\r");
            if (exception.Data.Count > 0)
            {
                lastStackTrace.Append("Data:");
                foreach (var item in exception.Data)
                {
                    var entry = (DictionaryEntry)item;
                    lastStackTrace.Append($"{entry.Key}: {exception.Data[entry.Key]}\n\r");
                }
            }
            //递归添加内部异常
            if ((ex = ex.InnerException) != null)
                return ExtractAllStackTrace(ex, lastStackTrace, ++exCount);
            return lastStackTrace;
        }
    }
}
